% One-dimensional equation of heat transfer for uniform medium

function [uu]=heat_1d_un(ud,kappa,a,nx,h,time,tau,b,gamma,bl,br,fbl,fbr,fs)

if bl == 0
    ud(1)=feval(fbl,0.);
end
if br == 0
    ud(nx)=feval(fbr,0.);
end
if gamma > 0. 

% Implicit scheme    
    if gamma < 0.5
        if b > 1./(2.-4.*gamma)
            disp('Scheme is unstable'); return;
        end    
    end    
    ba=2.*b;
    p(1:nx)=zeros(1,nx); s(1:nx)=zeros(1,nx); r(1:nx)=zeros(1,nx); d(1:nx)=zeros(1,nx);
    alfa(1:nx)=zeros(1,nx); beta(1:nx)=zeros(1,nx);
    if bl == 0
        s(1)=1.; r(1)=0.; d(1)=feval(fbl,time);
    else
        s(1)=1.+ba*gamma; r(1)=-ba*gamma;
        d(1)=(1.-ba*(1.-gamma))*ud(1)+ba*(1.-gamma)*ud(2)-...
                  ba*h*(gamma*feval(fbl,time)+(1.-gamma)*feval(fbl,time-tau))/a;
    end
    for n=2:nx-1
        x=(n-1)*h;
        p(n)=-b*gamma; s(n)=1.+ba*gamma; r(n)=-b*gamma;
        d(n)=ud(n)*(1.-ba*(1.-gamma))+b*(1.-gamma)*(ud(n+1)+ud(n-1))+...
                tau*feval(fs,x,time);
    end
    if br == 0
        p(nx)=0.; s(nx)=1.; d(nx)=feval(fbr,time);
    else
        s(nx)=1.+ba*gamma; p(nx)=-ba*gamma;
        d(nx)=(1.-ba*(1.-gamma))*ud(nx)+ba*(1.-gamma)*ud(nx-1)+...
                   ba*h*(gamma*feval(fbr,time)+(1.-gamma)*feval(fbr,time-tau))/a;
    end                  
    alfa(1)=-r(1)/s(1); beta(1)=d(1)/s(1);
    for n=2:nx
        aa=s(n)+p(n)*alfa(n-1);
        alfa(n)=-r(n)/aa; beta(n)=(d(n)-p(n)*beta(n-1))/aa;
    end
    uu(nx)=beta(nx);
    for n=nx-1:-1:1
        uu(n)=alfa(n)*uu(n+1)+beta(n);
    end
    return;
else    
    
% Explicit scheme
    if b > 0.5
        disp('Scheme is unstable'); return;
    end    
    ba=2.*b;
    if bl == 0
        uu(1)=feval(fbl,time);
    else
        uu(1)=(1.-ba)*ud(1)+ba*ud(2)-ba*h*feval(fbl,time-tau)/a;
    end
    for n=2:nx-1
        x=(n-1)*h;
        uu(n)=ud(n)*(1.-ba)+b*(ud(n+1)+ud(n-1))+tau*feval(fs,x,time);
    end
    if br == 0
        uu(nx)=feval(fbr,time);
    else
        uu(nx)=(1.-ba)*ud(nx)+ba*ud(nx-1)+ba*h*feval(fbr,time-tau)/a;
    end
    return;
end
return;