% Solve the Helmgoltz equation by the method of Concus and Golub

function [u,it]=helmgoltz_2d_cg(up,f,a,b,lx,ly,nx,ny,g1,g2,g3,g4)

hx=lx/(nx-1); hy=ly/(ny-1); hxs=1./(hx*hx); hys=1./(hy*hy);
f1=zeros(nx,ny); f2=zeros(nx,ny);
fs(1:ny)=zeros(1,nx); d=zeros(nx-2,ny);
x(1:nx)=((1:nx)-1.)*hx; y(1:ny)=((1:ny)-1.)*hy;

u(:,:)=up(:,:);
for n=1:nx
    for m=1:ny
        as(n,m)=feval(a,x(n),y(m));
    end
end
alfa=0.5*(max(max(as))+min(min(as)));
c1=hxs; c2=hys; c3=2.*(c1+c2)+alfa;
for m=1:ny
    u(1,m)=feval(g1,y(m))*sqrt(as(1,m)); u(nx,m)=feval(g3,y(m))*sqrt(as(nx,m));
end
for n=1:nx
    u(n,1)=feval(g2,x(n))*sqrt(as(n,1)); u(n,ny)=feval(g4,x(n))*sqrt(as(n,ny));
end
for n=2:nx-1
    for m=2:ny-1
        f1(n,m)=((sqrt(as(n-1,m))-2.*sqrt(as(n,m))+sqrt(as(n+1,m)))*hxs+...
                      (sqrt(as(n,m-1))-2.*sqrt(as(n,m))+sqrt(as(n,m+1)))*hys)/sqrt(as(n,m))-...
                       feval(b,x(n),y(m))/as(n,m);
        f2(n,m)=feval(f,x(n),y(m))/sqrt(as(n,m));          
    end
end
dif=1.; it=0;
while dif > 1.e-5
    it=it+1;
    for n=1:nx-2
        for m=1:ny
            d(n,m)=(f1(n+1,m)-alfa)*u(n+1,m)+f2(n+1,m);
        end
    end
    for m=1:ny
        d(1,m)=d(1,m)-c2*u(1,m); d(nx-2,m)=d(nx-2,m)-c2*u(nx,m);
    end
    for n=1:nx-2
        d(n,1)=u(n+1,1); d(n,ny)=u(n+1,ny);
    end
    d = fast_sin_transform(d);
    for k=1:nx-2
        p=c1; r=c1; s=-(c3-2.*c2*cos(pi*k/(nx-1)));
        for m=1:ny
            fs(m)=d(k,m);
        end
        fs=sweep(p,s,r,1.,0.,0.,1.,fs);
        d(k,1:ny)=fs(1:ny);
    end
    d = fast_sin_transform(d); v(2:nx-1,1:ny)=2.*d(1:nx-2,1:ny)/(nx-1);
    for m=1:ny
        v(1,m)=u(1,m); v(nx,m)=u(nx,m);
    end
    dif=norm(v-u,1)/norm(v,1); u(:,:)=v(:,:);
end
for n=1:nx
    for m=1:ny
        u(n,m)=u(n,m)/sqrt(as(n,m));
    end
end
return;